%  Copyright (C) 2010 Ville-Petteri Makinen
%
%  This program is free software; you can redistribute it and/or
%  modify it under the terms of the GNU General Public License
%  as published by the Free Software Foundation; either version 2
%  of the License, or (at your option) any later version.
%
%  This program is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU General Public License for more details.
%
%  You should have received a copy of the GNU General Public License
%  along with this program; if not, write to the
%  Free Software Foundation, Inc.,
%  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
%
%  CITATION:
%  Niemi J, Mäkinen VP, Heikkonen J, Tenkanen L, Hiltunen Y, Hannuksela ML,
%  Jauhiainen M, Forsblom C, Taskinen MR, Kesäniemi YA, Savolainen MJ,
%  Kaski K, Groop PH, Kovanen PT, Ala-Korpela M. Estimation of VLDL, IDL, LDL,
%  HDL2, apoA-I, and apoB from the Friedewald inputs--apoB and IDL, but not
%  LDL, are associated with mortality in type 1 diabetes. Ann Med.
%  2009;41(6):451-61. doi: 10.1080/07853890902893392. PMID: 19412820.
%

%  [y, yhdr] = extfried_exec(hdlc, chol, tg, unit);

% TG       M x 1 vector of triglyceride values.
% CHOL     M x 1 vector of cholesterol values.
% HDLC     M x 1 vector of hdl-cholesterol values.
% UNIT     Either 'mmol/l' or 'md/dl'.
% Y        M x K matrix of estimated lipoprotein variables.
% YHDR     1 x K cell array of variable names.

% Citation: Estimation of VLDL, IDL, LDL, HDL2, apoA-I, and apoB from the
% Friedewald inputs--apoB and IDL, but not LDL, are associated with mortality
% in type 1 diabetes. Niemi J, Mäkinen VP, Heikkonen J, Tenkanen L,
% Hiltunen Y, Hannuksela ML, Jauhiainen M, Forsblom C, Taskinen MR,
% Kesäniemi YA, Savolainen MJ, Kaski K, Groop PH, Kovanen PT, Ala-Korpela M.
% Ann Med. 2009;41(6):451-61. doi: 10.1080/07853890902893392.

function [y, yhdr] = extfried_exec(hdlc, chol, tg, unit);

% Convert to mmol/l.
switch unit
case 'mmol/l'
case 'mg/dl'
  hdlc=hdlc.*0.02586;
  chol=chol.*0.02586;
  tg=tg.*0.01129;
otherwise
  error('Unknown concentration unit.');
end

%limits for inputs
limits_lipo= [.31 4.95 1.93 12.51 .11 9.6];
limits_hdl2= [.67 4.95 2.49 10.64 .11 9.6];
limits_apo= [.34 2.64 1.71 9.13 .45 6.6];
h=[hdlc hdlc hdlc hdlc hdlc hdlc];
c=[chol chol chol chol chol chol];
t=[tg tg tg tg tg tg];

%NaNs for lipos
for j=1:size(tg,1)
    if(hdlc(j)<limits_lipo(1) | hdlc(j)>limits_lipo(2))
        h(j,1:3)=NaN;
    end
    if(hdlc(j)<limits_hdl2(1) | hdlc(j)>limits_hdl2(2))
        h(j,4)=NaN;
    end
    if(hdlc(j)<limits_apo(1) | hdlc(j)>limits_apo(2))
        h(j,5:6)=NaN;
    end

    if(chol(j)<limits_lipo(3) | chol(j)>limits_lipo(4))
        c(j,1:3)=NaN;
    end
    if(chol(j)<limits_hdl2(3) | chol(j)>limits_hdl2(4))
        c(j,4)=NaN;
    end
    if(chol(j)<limits_apo(3) | chol(j)>limits_apo(4))
        c(j,5:6)=NaN;
    end
    
    if(tg(j)<limits_lipo(5) | tg(j)>limits_lipo(6))
        t(j,1:3)=NaN;
    end
    if(tg(j)<limits_hdl2(5) | tg(j)>limits_hdl2(6))
        t(j,4)=NaN;
    end
    if(tg(j)<limits_apo(5) | tg(j)>limits_apo(6))
        t(j,5:6)=NaN;
    end
end

load mlpcom_extfrie.oct;
DataSize=size(mlpcom,2)-1;
ComSize=50;
%Scaling & predicting
for oindex=1:DataSize
  koldata=[h(:,oindex) c(:,oindex) t(:,oindex)];
  mlpc=mlpcom{oindex};
  [pp]=scale(koldata,mlpc(1).pmag,mlpc(1).poffset);
  tye=zeros(size(koldata,1),1);
  for i=1:ComSize    
    net=mlpc(i).net;
    tye=tye + mlp2fwd(net,pp);
  end
  y(:,oindex)=tye./ComSize;
end
yhdr = {'VLDL-TG', 'IDL-C', 'LDL-C', 'HDL2-C', 'ApoA-I', 'ApoB'};

% Convert back to original unit.
switch unit
case 'mmol/l'
case 'mg/dl'
  [dummy, tgvars] = cellisect(yhdr, {'VLDL-TG'});
  [dummy, chvars] = cellisect(yhdr, {'IDL-C', 'LDL-C', 'HDL2-C'});
  y(:, tgvars) = y(:, tgvars)./0.01129;
  y(:, chvars) = y(:, chvars)./0.02586;
otherwise
  error('Unknown concentration unit.');
end

% Check if values are OK.
if sum(0*y(:) == 0) < 1
  error('Could not calculate any values (wrong concentration unit?).');
end

return;

%--------------------------------------------------------------------------

function [y, z] = mlp2fwd(net, x)
ndata = size(x, 1);
z = tanh(x*net.w1 + ones(ndata, 1)*net.b1);
y = z*net.w2 + ones(ndata, 1)*net.b2;
return;

%--------------------------------------------------------------------------

function [a,mag,offset]=scale(b,range,offset)
% [a,mag,offset]=scale(b,range)
%    OR
% a=scale(b,mag,offset)
%
%   Scales each column of b to be in range [range(1) range(2)]
%   Default range is [0 1]
%
%   If range is scalar S, then standard deviation of each column is
%   set to S and mean to zero. Thus scale(b,1) is the basi scaling.
%
%   Mag and offset return the scaling coefficients so that 
%       [n,m]=size(a);
%       b=a./(ones(n,1)*mag) + ones(n,1)*offset
%   The latter form allows to use the same coefficients in subsequent
%	scalings.
%
%   See rescale for rescaling the data back to original range
%
[n,m]=size(b);
if nargin<2, range=[0 1]; end
if nargin==3,
  mag=range;
  for i=1:m,
    a(:,i)=(b(:,i)-offset(i))*mag(i);
  end
else
  if max(size(range))==1,
    mag=range./std(b);
    offset=mean(b);
    a=zeros(size(b));
    for i=1:m,
      r=(max(b(:,i))-offset(i));
      if r==0,
	a(:,i)=zeros(size(b(:,i)));
	mag(i)=1;
      else
	a(:,i)=(b(:,i)-offset(i))*mag(i);
      end
    end
  else
    X=range(2)-range(1);
    offset=min(b);
    mag=offset;
    a=zeros(size(b));
    for i=1:m,
      r=(max(b(:,i))-offset(i));
      if r==0,
	a(:,i)=zeros(size(b(:,i)));
	mag(i)=1; off(i)=0;
      else
	mag(i)=X/r;
	offset(i)=offset(i)-range(1)/mag(i) ;
	a(:,i)=(b(:,i)-offset(i))*mag(i);
      end
    end
  end
end

return;
